@@ -18,14 +18,13 @@ import inspect
 import locale
 import os.path
 import shutil
-import struct
 import sys
-import tempfile
 import threading
 
 from . import engines
 from . import errors
 from . import ipc
+from . import temporary
 from . import text_zones
 from . import utils
 from . import version
@@ -36,20 +35,13 @@ import djvu.decode
 
 __version__ = version.__version__
 
-bitonal_pixel_format = djvu.decode.PixelFormatPackedBits('>')
-bitonal_pixel_format.rows_top_to_bottom = 1
-bitonal_pixel_format.y_top_to_bottom = 1
-
-rgb_pixel_format = djvu.decode.PixelFormatRgb()
-rgb_pixel_format.rows_top_to_bottom = 1
-rgb_pixel_format.y_top_to_bottom = 1
-
 system_encoding = locale.getpreferredencoding()
 
 class Saver(object):
 
     in_place = False
 
+    @utils.not_overridden
     def save(self, document, pages, djvu_path, sed_file):
         raise NotImplementedError
 
@@ -275,21 +267,17 @@ class ArgumentParser(argparse.ArgumentParser):
 class Context(djvu.decode.Context):
 
     def init(self, options):
-        self._temp_dir = tempfile.mkdtemp(prefix='ocrodjvu.')
+        self._temp_dir = temporary.raw.mkdtemp(prefix='ocrodjvu.')
         self._debug = options.debug
         self._options = options
-        if self._options.render_layers == djvu.decode.RENDER_MASK_ONLY:
-            self._pixel_format = bitonal_pixel_format
-        else:
-            self._pixel_format = rgb_pixel_format
-        if self._options.engine.image_format == 'bmp':
-            self._pixel_format.rows_top_to_bottom = 0
+        bpp = 24 if self._options.render_layers != djvu.decode.RENDER_MASK_ONLY else 1
+        self._image_format = self._options.engine.image_format(bpp)
 
     def _temp_file(self, name, auto_remove=True):
         path = os.path.join(self._temp_dir, name)
         file = open(path, 'w+b')
         if not self._debug and auto_remove:
-            file = tempfile._TemporaryFileWrapper(file, file.name)
+            file = temporary.wrapper(file, file.name)
         return file
 
     def handle_message(self, message):
@@ -301,56 +289,11 @@ class Context(djvu.decode.Context):
     def get_output_image(self, nth, page_job):
         size = page_job.size
         rect = (0, 0) + size
-        output_format = self._engine.image_format
-        file = self._temp_file('%06d.%s' % (nth, output_format))
+        output_format = self._image_format
+        file = self._temp_file('%06d.%s' % (nth, output_format.extension))
         try:
-            if output_format == 'ppm':
-                if self._pixel_format.bpp == 1:
-                    file.write('P4 %d %d\n' % size)  # PBM header
-                else:
-                    file.write('P6 %d %d 255\n' % size)  # PPM header
-                data = page_job.render(
-                    self._options.render_layers,
-                    rect, rect,
-                    self._pixel_format
-                )
-                file.write(data)
-                file.flush()
-            elif output_format == 'bmp':
-                dpm = int(page_job.dpi * 39.37 + 0.5)
-                data = page_job.render(
-                    self._options.render_layers,
-                    rect, rect,
-                    self._pixel_format,
-                    row_alignment=4,
-                )
-                n_palette_colors = 2 * (self._pixel_format.bpp == 1)
-                headers_size = 54 + 4 * n_palette_colors
-                file.write(struct.pack('<ccIHHI',
-                    'B', 'M', # magic
-                    len(data) + headers_size, # whole file size
-                    0, 0, # identification magic
-                    headers_size # offset to pixel data
-                ))
-                file.write(struct.pack('<IIIHHIIIIII',
-                    40, # size of this header
-                    size[0], size[1], # image size in pixels
-                    1, # number of color planes
-                    self._pixel_format.bpp, # number of bits per pixel
-                    0, # compression method
-                    len(data), # size of pixel data
-                    dpm, dpm, # resolution in pixels/meter
-                    n_palette_colors, # number of colors in the color pallete
-                    n_palette_colors # number of important colors
-                ))
-                if self._pixel_format.bpp == 1:
-                    # palette:
-                    file.write(struct.pack('<BBBB', 0, 0, 0, 0))
-                    file.write(struct.pack('<BBBB', 0xff, 0xff, 0xff, 0))
-                file.write(data)
-                file.flush()
-            else:
-                raise NotImplementedError('Unknown image format: %s' % output_format)
+            output_format.write_image(page_job, self._options.render_layers, file)
+            file.flush()
             yield file
         finally:
             file.close()
